import 'package:http/http.dart' as http;
import 'dart:io';
import 'dart:convert';
import 'dart:async';

class NetworkManager {

  //GET API CALL
  Future<Map<String, dynamic>> getAPICall(String url,
      {Map<String, String> header}) async {
    var responseJson;
    try {
      final response = await http.get(url, headers: header);
      print('original response $response');
      responseJson = _response(response);
    } on SocketException {
      throw FetchDataException('No Internet connection');
    }
    return responseJson;
  }

  //POST API CALL
  Future<Map<String, dynamic>> postAPICall(String url,
      {Map body, Map<String, String> header}) async {
    var responseJson;
    try {
      final strFullURL = url;
      print(strFullURL);
      print(body);
      final http.Response response = await http.post(
        strFullURL,
        headers: header ??
            <String, String>{
              'Content-Type': 'application/json',
            },
        body: jsonEncode(body),
      );
      responseJson = _response(response);
    } on SocketException {
      throw FetchDataException('No Internet connection');
    } catch (e) {
      e.toString();
    }
    return responseJson;
  }

  // PUT API CALL
  Future<Map<String, dynamic>> putAPICall(String url,
      {Map body, Map<String, String> header}) async {
    var responseJson;
    try {
      final strFullURL = url;
      print(strFullURL);
      print(body);
      final http.Response response = await http.put(
        strFullURL,
        headers: header,
        body: jsonEncode(body),
      );
      responseJson = _response(response);
    } on SocketException {
      throw FetchDataException('No Internet connection');
    } catch (e) {
      e.toString();
    }
    return responseJson;
  }

  //DELETE API CALL
  Future<Map<String, dynamic>> deleteAPICall(String _url, {Map<String, String> header, Map bodyParameters}) async {
    var responseJson;
    try {

      final url = Uri.parse(_url);
      final request = http.Request("DELETE", url);
      request.headers.addAll(header);
      request.body = jsonEncode(bodyParameters);
      final response = await request.send();
      print('original response $response');
      var httpResponse = await http.Response.fromStream(response);
      responseJson = _response(httpResponse);
    } on SocketException {
      throw FetchDataException('No Internet connection');
    }
    return responseJson;
  }

  Map<String, dynamic> _response(http.Response response) {
    switch (response.statusCode) {
      case 200:
        var responseJson = <String, dynamic>{};
        var convertDataToJson = json.decode(response.body);
        responseJson = convertDataToJson;
        print('its code 200');
        //print(responseJson);
        return responseJson;
      case 400:
        print('status code $response.statusCode');
        print('response $response');

        throw BadRequestException(response.body.toString());

      case 403:
        print('status code $response.statusCode');
        print('response $response');

        throw UnauthorisedException(response.body.toString());

      case 500:
        throw ServerError(response.body.toString());

      default:
        final code = response.statusCode;
        print('response $code');
        print(response);

        var responseJson = <String, dynamic>{};
        var convertDataToJson = json.decode(response.body);
        responseJson = convertDataToJson;
        print(responseJson);
        return responseJson;

    //throw FetchDataException(
    //  'Error occurred while Communication with Server with StatusCode : ${response.statusCode}');
    }
  }
}

class CustomException implements Exception {
  final _message;
  final _prefix;

  CustomException([this._message, this._prefix]);

  String toString() {
    return "$_prefix$_message";
  }
}

class FetchDataException extends CustomException {
  FetchDataException([String message])
      : super(message, "Error During Communication: ");
}

class BadRequestException extends CustomException {
  BadRequestException([message]) : super(message, "Invalid Request: ");
}

class UnauthorisedException extends CustomException {
  UnauthorisedException([message]) : super(message, "Unauthorised: ");
}

class InvalidInputException extends CustomException {
  InvalidInputException([String message]) : super(message, "Invalid Input: ");
}

class ServerError extends CustomException {
  ServerError([String message]) : super(message, "Internal Server error: ");
}
